package huettester;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.Boolean;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.huet.Configuration;
import org.sanskritlibrary.huet.Constants;
import org.sanskritlibrary.huet.Extraction;
import org.sanskritlibrary.huet.Extractions;
import org.sanskritlibrary.huet.GrammarType;
import org.sanskritlibrary.huet.HTMLParser;
import org.sanskritlibrary.huet.PadaStemRoot;
import org.sanskritlibrary.huet.ScreenScraper;
import org.sanskritlibrary.huet.Utils;
import org.sanskritlibrary.huet.Version1Fixup;

/*
 * Try meyaH
*/


public class HeritageReader {
    Configuration config;

    public static void main(String[] args) throws Exception {
        new HeritageReader (args);
    }

    public HeritageReader (String[] args) throws Exception {
        boolean createHelp = true;

        if (args.length == 0) {
            System.out.println("You forgot the extraction ID\n");
            return;
        }

        if (args[0].equals("xx")) {
            System.out.println("Converting version 1");
            convertVersion1("/Users/ralph/Downloads/PT-sentences/");

            return;
        }

        Utils.transcoderDirectory = "transcoders";

        config = new Configuration ("config.xml", ".");
        Extractions extractions = new Extractions("extractions.xml");

        if (createHelp) {
            // The inflections are stored in config.xml and were parsed by the Configuration object
            HTMLParser hp;
            BufferedWriter bw = IOAssistance.openWriter("helpmorphids.html");
            bw.write("<html><head><meta http-equiv='content-type' content='text/html;charset=UTF-8' /><title></title></head><body>");

            GrammarType.rejected = IOAssistance.openWriter("rejected.txt"); // where Configuration object writes rejected combinations
            if (!GrammarType.debugHTML) {
                bw.write(config.getNominals());             // HTML for nominal table
                bw.write(config.getIndeclinables());        // HTML for indeclinable table
            }
            bw.write(config.getVerbs());    // HTML for verb table.
            GrammarType.rejected.close();
            bw.write("\n</body></html>");
            bw.close();

            System.out.format("%d nouns + %d indeclinables + %d verbs = %d\n",
                    GrammarType.totalNouns, GrammarType.totalIndeclinables, GrammarType.totalVerbs,
                    GrammarType.totalNouns + GrammarType.totalIndeclinables + GrammarType.totalVerbs);

            System.out.format("%d legal, %d rejected, %d duplicates = %d\n", GrammarType.legal, GrammarType.constraintFailed, GrammarType.duplicates,
                    GrammarType.legal + GrammarType.constraintFailed + GrammarType.duplicates);

            hp = new HTMLParser("helpmorphids.html");
            FileOutputStream fos = new FileOutputStream("morphids.obj");
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(hp);
            oos.close();
            System.out.format("#duplicates removed: %d\n", GrammarType.duplicates);
            config.tagMapSHtoSL = hp.getSHtoSL();
            config.tagMapSHtoUH = hp.getSHtoUH();

            InflectionTemplate it = new InflectionTemplate("inflectionTemplate.xml", config, extractions.getExtraction(args[0]).inputEncoding);
            it.createTable("inflectionsSH.html", "SH", false);
            it.createTable("inflectionsSL.html", "SL", false);
            it.createTable("inflectionsUH.html", "UH", true);
        }

        ScreenScraper scraper = new ScreenScraper(config);
        DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
        Date date = new Date();
        System.out.println(dateFormat.format(date));
        Extraction extraction = extractions.getExtraction(args[0]);
        PadaStemRoot.targetEncoding = extraction.displayEncoding;

        scraper.setCorpus(args[0]);
        scraper.doScreenScrape(extraction);
        dateFormat = new SimpleDateFormat("HH:mm:ss");
        date = new Date();
        System.out.println(dateFormat.format(date));

        ArrayList<String> a = scraper.getNotMapped();
        for (String s : a) {
            System.out.println(s);
        }
    }

    private void convertVersion1(String dir) throws Exception {
        Hashtable<String, Boolean> h = new Hashtable<String, Boolean>();
        Regex re = new Regex("", "PT-sentence-(\\d+)");
        String xmldir = String.format("%sxml", dir);
        File fxmldir = new File(xmldir);
        fxmldir.mkdirs();
        File fdir = new File(dir);
        File[] files = fdir.listFiles();
        for (File file : files) {
            String filename = file.getName();
            Version1Fixup v1 = null;
            if (filename.endsWith(".txt")) {
                try {
                v1 = new Version1Fixup(file.getAbsolutePath(), "roman", "SL", "1.0");
                } catch (Exception ex) {
                    System.out.format("%s not well-formed\n", filename);
                    continue;
                }
                re.reset(filename);
                if (re.find()) {
                    String number = re.getGroup(1);
                    h.put(number, Boolean.TRUE);
                    String outfile = String.format("%s/sentence%s.xml", xmldir, number);
                    BufferedWriter bw = IOAssistance.openWriter(outfile);
                    String xml = v1.getXML();
                    bw.write(xml);
                    bw.close();
                }
            }
        }

        ArrayList<Integer> a = new ArrayList<Integer>();
        Enumeration<String> en = h.keys();
        while (en.hasMoreElements()) {
            String number = en.nextElement();
            a.add(Integer.parseInt(number));
        }
        Collections.sort(a);
        StringBuilder sb = new StringBuilder();
        int current = 1;

        for (Integer n : a) {
            while (n > current) {
                sb.append("u");
                current++;
            }
            sb.append("d");
            current++;
        }
        String outfile = String.format("%s/indexSupp.txt", xmldir);
        BufferedWriter bw = IOAssistance.openWriter(outfile);
        bw.write(sb.toString());
        bw.close();
    }
}
