package huettester;

import java.io.BufferedWriter;
import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.sanskritlibrary.SAXParserHandler;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.huet.Configuration;
import org.sanskritlibrary.huet.GrammarType;
import org.sanskritlibrary.huet.Utils;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class InflectionTemplate extends DefaultHandler implements SAXParserHandler, Serializable {
    transient Locator locator;
    transient ArrayList<String> pathStack = new ArrayList<String>();
    transient Hashtable<String, StringBuilder> buffers = new Hashtable<String, StringBuilder>();
    String header;
    Hashtable<String, String> h = new Hashtable<String, String>();
    String trailer;
    Configuration config;
    String inputEncoding;
    transient int colCount = 0;
    transient String valueSL;
    transient String valueSH;
    transient String valueUH;

    public InflectionTemplate (String fileName, Configuration config, String inputEncoding) throws Exception
    {
        this.config = config;
        this.inputEncoding = inputEncoding;
        // Use the default (non-validating) parser
        SAXParserFactory factory = SAXParserFactory.newInstance();

        // Parse the input
        SAXParser saxParser = factory.newSAXParser();
        try {
            saxParser.parse( new File(fileName), this);
        } catch (Exception ex) {
            throw ex;
        }
    }

    public void createTable(String tableName, String type, boolean usesSanskrit) throws Exception {
        BufferedWriter bw = IOAssistance.openWriter(tableName);
        bw.write(h.get("t.header"));
        // Nominals
        write (bw, config.nominalGender,"gender", type, 0, usesSanskrit);
        write (bw, config.nominalCase, "case", type, 0, usesSanskrit);
        write (bw, config.nominalNumber, "nounNumber", type, 0, usesSanskrit);

        // Indeclinable
        write (bw, config.indeclinable, "indeclinable", type, 0, usesSanskrit);

        // Verbs
        write (bw, config.verbsConjugation, "conjugation", type, 1, usesSanskrit);
        write (bw, config.verbsTenseMoodAspect, "tense", type, 0, usesSanskrit);
        write (bw, config.verbsClass, "class", type, 0, usesSanskrit);
        write (bw, config.verbsVoice, "voice", type, 0, usesSanskrit);
        write (bw, config.verbsPerson, "person", type, 0, usesSanskrit);
        write (bw, config.verbsNumber, "verbNumber", type, 0, usesSanskrit);

        bw.write(h.get("t.trailer"));
        bw.close();
    }

    private void write (BufferedWriter bw, ArrayList<GrammarType> gts, String templateName, String type, int start, boolean usesSanskrit) throws Exception {
        Hashtable<String, Boolean> dups = new Hashtable<String, Boolean>();
        String before = h.get(String.format("t.%s.before", templateName));
        if (before == null) {
            before = null;
        }
        bw.write(before);

        String insert = h.get(String.format("t.%s.insertRow", templateName));
        String insertValue = null;
        String insertAfter = null;
        if (insert != null) {
            insert = insert.trim();
            String[] fields = insert.split(":", 2);
            if (fields.length == 2) {
                insertAfter = fields[0];
                insertValue = fields[1];
            }
        }
        int len = gts.size();
        int rownum = 1;
        String firstrowTemplate = h.get(String.format("t.%s.firstrow", templateName));
        if (firstrowTemplate != null) {
            bw.write(String.format(firstrowTemplate, rownum, rownum));
            rownum++;
        }
        String rowTemplate = h.get(String.format("t.%s.row", templateName));
        for (int i=start; i<len; i++) {
            GrammarType gt = gts.get(i);
            String value = gt.value.get(type);
            if (value == null) {
                value = "?";
            } else if (!inputEncoding.equals("slp1") && usesSanskrit) {
                String value2 = Utils.convertWithPassThrough(value, "slp1", inputEncoding);
                if (!value2.equals(value)) {
                    value2 = value;
                }
            }

            if (dups.get(value) != null) {
                continue;
            } else {
                dups.put(value, Boolean.TRUE);
            }
            try {
                bw.write(String.format(rowTemplate, rownum, rownum, value));
                if (insertValue != null) {
                    String key = gt.value.get("SL");
                    if (key != null && key.equals(insertAfter)) {
                        bw.write(insertValue);
                    }
                }

                rownum++;
            } catch (Exception ex) {
                System.out.format("%s\n", ex.getMessage());
            }
        }
        bw.write(h.get(String.format("t.%s.after", templateName)));
    }


    private void push (String qName) {
        pathStack.add(qName);
    }

    private String pop () {
        int last = pathStack.size()-1;
        String top = null;
        if (last >= 0) {
            top = pathStack.get(last);
            pathStack.remove(last);
        }

        return top;
    }

    private String peek () {
        int last = pathStack.size()-1;
        String top = null;
        if (last >= 0) {
            top = pathStack.get(last);
        }

        return top;

    }

    private String buildPath () {
        StringBuilder sb = new StringBuilder();
        String delim = "";
        for (String name : pathStack) {
            sb.append(String.format("%s%s", delim, name));
            delim = ".";
        }
        return sb.toString();
    }

    private void displayError(String msg) {
        System.out.format("Line %d: %s\n", locator.getLineNumber(), msg);
    }

    @Override
    public void warning(SAXParseException exception) throws SAXException {
        displayError(exception.getMessage());
    }

    @Override
    public void error(SAXParseException exception) throws SAXException {
        displayError(exception.getMessage());
    }

    @Override
    public void fatalError(SAXParseException exception) throws SAXException {
        displayError(exception.getMessage());
    }

    @Override
    public void setDocumentLocator(Locator locator) {
        this.locator = locator;
    }

    @Override
    public void startDocument() throws SAXException {
    }

    @Override
    public void endDocument() throws SAXException {
    }

    @Override
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
    }

    @Override
    public void endPrefixMapping(String prefix) throws SAXException {
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        push(qName);
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        if (qName.equals("insertRow")) {
            qName = "insertRow";
        }
        String path = buildPath();
        StringBuilder sb = buffers.get(path);
        if (sb == null) {
            h.put(path, "");
        } else {
            String text = sb.toString().replaceAll("\n\n", "\n");
            while (text.contains("\n\n")) {
                text = text.replaceAll("\n\n", "\n");
            }
            h.put(path, text);
        }
        pop();
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        String path = buildPath();
        StringBuilder sb = buffers.get(path);
        if (sb == null) {
            sb = new StringBuilder();
            buffers.put(path, sb);
        }
        sb.append(new String(ch, start, length));
    }
}
